@@ -24,33 +24,44 @@
 
 #include <gtk/gtk.h>
 
+#ifdef GIO_ENABLED
+#include <gio/gio.h>
+#endif
+
 #ifndef _COMMON_H
 #define _COMMON_H
 
-#define CACHE_SIZE		"/apps/gnome-mplayer/preferences/cache_size"
-#define OSDLEVEL		"/apps/gnome-mplayer/preferences/osdlevel"
-#define PPLEVEL		"/apps/gnome-mplayer/preferences/pplevel"
-#define SOFTVOL			"/apps/gnome-mplayer/preferences/softvol"
-#define VERBOSE			"/apps/gnome-mplayer/preferences/verbose"
-#define VERTICAL		"/apps/gnome-mplayer/preferences/vertical"
-#define FORCECACHE		"/apps/gnome-mplayer/preferences/forcecache"
-#define LAST_DIR		"/apps/gnome-mplayer/preferences/last_dir"
-#define SHOWPLAYLIST	"/apps/gnome-mplayer/preferences/showplaylist"
-#define DISABLEASS          "/apps/gnome-mplayer/preferences/disableass"
-#define DISABLEEMBEDDEDFONTS    "/apps/gnome-mplayer/preferences/disableembeddedfonts"
-#define DISABLEDEINTERLACE	"/apps/gnome-mplayer/preferences/disable_deinterlace"
-#define DISABLEFRAMEDROP	"/apps/gnome-mplayer/preferences/disableframedrop"
-#define DISABLEFULLSCREEN	"/apps/gnome-mplayer/preferences/disablefullscreen"
-#define DISABLECONTEXTMENU	"/apps/gnome-mplayer/preferences/disablecontextmenu"
-#define DISABLEPAUSEONCLICK	"/apps/gnome-mplayer/preferences/disable_pause_on_click"
-#define METADATACODEPAGE	"/apps/gnome-mplayer/preferences/metadatacodepage"
-#define SUBTITLEFONT	"/apps/gnome-mplayer/preferences/subtitlefont"
-#define SUBTITLESCALE	"/apps/gnome-mplayer/preferences/subtitlescale"
-#define SUBTITLECODEPAGE	"/apps/gnome-mplayer/preferences/subtitlecodepage"
-#define SUBTITLECOLOR	"/apps/gnome-mplayer/preferences/subtitlecolor"
+#define MIXER			"mixer"
+#define CACHE_SIZE		"cache_size"
+#define OSDLEVEL		"osdlevel"
+#define PPLEVEL		"pplevel"
+#define SOFTVOL			"softvol"
+#define VERBOSE			"verbose"
+#define VERTICAL		"vertical"
+#define FORCECACHE		"forcecache"
+#define LAST_DIR		"last_dir"
+#define SHOWPLAYLIST	"showplaylist"
+#define SHOWDETAILS	"showdetails"
+#define DISABLEASS          "disableass"
+#define DISABLEEMBEDDEDFONTS    "disableembeddedfonts"
+#define DISABLEDEINTERLACE	"disable_deinterlace"
+#define DISABLEFRAMEDROP	"disableframedrop"
+#define DISABLEFULLSCREEN	"disablefullscreen"
+#define DISABLECONTEXTMENU	"disablecontextmenu"
+#define DISABLEPAUSEONCLICK	"disable_pause_on_click"
+#define DISABLEANIMATION	"disable_animation"
+#define METADATACODEPAGE	"metadatacodepage"
+#define SUBTITLEFONT	"subtitlefont"
+#define SUBTITLESCALE	"subtitlescale"
+#define SUBTITLECODEPAGE	"subtitlecodepage"
+#define SUBTITLECOLOR	"subtitlecolor"
+#define VOLUME	"volume"
+#define USE_MEDIAKEYS		"use_mediakeys"
+#define DISABLE_COVER_ART_FETCH "disable_cover_art_fetch"
+#define FULLSCREEN	"fullscreen"
 
-#define MPLAYER_BIN		"/apps/gnome-mplayer/preferences/mplayer_bin"
-#define EXTRAOPTS		"/apps/gnome-mplayer/preferences/extraopts"
+#define MPLAYER_BIN		"mplayer_bin"
+#define EXTRAOPTS		"extraopts"
 
 #define DISABLE_QT		"/apps/gecko-mediaplayer/preferences/disable_qt"
 #define DISABLE_REAL	"/apps/gecko-mediaplayer/preferences/disable_real"
@@ -58,10 +69,17 @@
 #define DISABLE_DVX		"/apps/gecko-mediaplayer/preferences/disable_dvx"
 #define DISABLE_EMBEDDING		"/apps/gecko-mediaplayer/preferences/disable_embedding"
 
-#define REMEMBER_LOC		"/apps/gnome-mplayer/preferences/remember_loc"
-#define WINDOW_X		"/apps/gnome-mplayer/preferences/window_x"
-#define WINDOW_Y		"/apps/gnome-mplayer/preferences/window_y"
+#define REMEMBER_LOC		"remember_loc"
+#define WINDOW_X		"window_x"
+#define WINDOW_Y		"window_y"
+#define WINDOW_HEIGHT	"window_height"
+#define WINDOW_WIDTH	"window_width"
 
+#define KEEP_ON_TOP		"keep_on_top"
+#define SINGLE_INSTANCE "single_instance"
+#define REPLACE_AND_PLAY "replace_and_play"
+#define SHOW_NOTIFICATION "show_notification"
+#define SHOW_STATUS_ICON "show_status_icon"
 
 // JavaScript Playstates
 #define STATE_UNDEFINED     0
@@ -97,8 +115,11 @@ PLAYBACK_ERROR playback_error;
 
 typedef struct _IdleData {
     gchar info[1024];
+    gchar display_name[1024];
     gchar media_info[2048];
+    gchar media_notification[2048];
     gchar url[1024];
+    gint windowid;
     gchar *device;
     gdouble percent;
     gdouble cachepercent;
@@ -113,17 +134,20 @@ typedef struct _IdleData {
     gint last_y;
     gint width;
     gint height;
-    gint videopresent;
+    gboolean videopresent;
+    gboolean audiopresent;
     gboolean fullscreen;
     gboolean showcontrols;
     gdouble position;
     gdouble length;
-    gint byte_pos;
+    glong byte_pos;
+    gint chapters;
     gint brightness;
     gint contrast;
     gint gamma;
     gint hue;
     gint saturation;
+    gchar demuxer[64];
     gchar video_format[64];
     gchar video_codec[16];
     gchar video_fps[16];
@@ -134,12 +158,27 @@ typedef struct _IdleData {
     gchar audio_channels[16];
     gchar metadata[1024];
     gboolean fromdbus;
+    gboolean window_resized;
+    gboolean has_chapters;
+    gboolean tmpfile;
+    gboolean sub_visible;
+    gint sub_demux;
+    gint switch_audio;
+    gboolean retry_on_full_cache;
+#ifdef GIO_ENABLED
+    GFile *src;
+    GFile *tmp;
+    GCancellable *cancel;
+    GMutex *caching;
+	GCond *caching_complete;
+#endif
 } IdleData;
 
 IdleData *idledata;
 
 
 typedef struct _ThreadData {
+    gchar uri[2048];
     gchar filename[1024];
     gchar subtitle[1024];
     gint streaming;
@@ -154,11 +193,38 @@ enum {
     COUNT_COLUMN,
     PLAYLIST_COLUMN,
     ARTIST_COLUMN,
+    ALBUM_COLUMN,
     LENGTH_COLUMN,
+    LENGTH_VALUE_COLUMN,
     SUBTITLE_COLUMN,
+    COVERART_COLUMN,
+    AUDIO_CODEC_COLUMN,
+    VIDEO_CODEC_COLUMN,
+    DEMUXER_COLUMN,
+    VIDEO_WIDTH_COLUMN,
+    VIDEO_HEIGHT_COLUMN,
     N_COLUMNS
 };
 
+typedef struct _MetaData {
+    gchar *title;
+    gchar *artist;
+    gchar *album;
+    gchar *length;
+    gfloat length_value;
+    gchar *subtitle;
+    gchar *audio_codec;
+    gchar *video_codec;
+    gchar *demuxer;
+    gint width;
+    gint height;
+} MetaData;
+
+typedef struct _LangMenu {
+    gchar *label;
+    int value;
+} LangMenu;
+
 //Define MIME for DnD
 #define DRAG_NAME_0		"text/plain"
 #define DRAG_INFO_0		0
@@ -174,6 +240,7 @@ gint osdlevel;
 gint pplevel;
 gint streaming;
 gint showcontrols;
+gboolean showsubtitles;
 gint fullscreen;
 gint videopresent;
 gint playlist;
@@ -193,6 +260,7 @@ gint stored_window_width, stored_window_height;
 gchar vm[10];
 gchar *vo;
 gchar *ao;
+gchar *mixer;
 gboolean disable_deinterlace;
 gboolean disable_framedrop;
 gboolean disable_context_menu;
@@ -207,7 +275,6 @@ gint js_state;
 gchar *rpname;
 gchar *rpconsole;
 gchar *rpcontrols;
-gboolean show_media_label;
 gboolean control_instance;
 gchar *playlistname;
 gboolean ok_to_play;
@@ -218,9 +285,14 @@ gchar *metadata_codepage;
 gint volume;
 gboolean vertical_layout;
 gboolean playlist_visible;
+gboolean details_visible;
 gboolean restore_playlist;
+gboolean restore_details;
+gboolean restore_info;
+gint restore_pane;
 gboolean disable_ass;
 gboolean disable_embeddedfonts;
+gboolean disable_animation;
 gchar *subtitlefont;
 gdouble subtitle_scale;
 gchar *subtitle_codepage;
@@ -229,10 +301,21 @@ gboolean quit_on_complete;
 gchar *mplayer_bin;
 gchar *extraopts;
 gboolean single_instance;
+gboolean new_instance;
+gboolean replace_and_play;
+gboolean use_pausing_keep_force;
+gboolean show_notification;
+gboolean show_status_icon;
+gboolean load_tracks_from_gpod;
+gchar *gpod_mount_point;
+gboolean disable_cover_art_fetch;
 
 gboolean remember_loc;
 gint loc_window_x;
 gint loc_window_y;
+gint loc_window_height;
+gint loc_window_width;
+gboolean keep_on_top;
 
 // tv stuff
 gchar *tv_device;
@@ -244,6 +327,9 @@ gint tv_fps;
 
 GThread *thread;
 GMutex *slide_away;
+GCond *mplayer_complete_cond;
+
+gboolean use_mediakeys;
 
 gboolean qt_disabled;
 gboolean real_disabled;
@@ -260,11 +346,13 @@ GtkWidget *list;
 
 GtkWidget *create_window(gint windowid);
 gint get_player_window();
+void adjust_paned_rules();
+void reset_paned_rules();
 gboolean hide_buttons(void *data);
 gboolean show_copyurl(void *data);
 gboolean set_media_info(void *data);
-gboolean set_media_info_name(gchar * filename);
 gboolean set_media_label(void *data);
+gboolean set_cover_art(gpointer pixbuf);
 gboolean set_progress_value(void *data);
 gboolean set_progress_text(void *data);
 gboolean set_progress_time(void *data);
@@ -284,7 +372,14 @@ gboolean set_position(void *data);
 gboolean set_volume(void *data);
 gboolean set_fullscreen(void *data);
 gboolean set_show_controls(void *data);
+gboolean get_show_controls();
 gboolean set_window_visible(void *data);
+gboolean set_update_gui(void *data);
+gboolean set_subtitle_visibility(void *data);
+gboolean set_item_add_info(void *data);
+void remove_langs(GtkWidget * item, gpointer data);
+gboolean set_new_lang_menu(gpointer data);
+gboolean set_new_audio_menu(gpointer data);
 gboolean make_panel_and_mouse_invisible(gpointer data);
 void make_button(gchar * src, gchar * href);
 void dbus_open_by_hrefid(gchar * hrefid);
@@ -300,10 +395,14 @@ void dbus_unhook();
 void dbus_enable_screensaver();
 void dbus_disable_screensaver();
 
-void shutdown();
+void mplayer_shutdown();
 gpointer launch_player(gpointer data);
 
 gboolean update_mplayer_config();
 gboolean read_mplayer_config();
 
+#ifdef GTK2_12_ENABLED
+GtkRecentManager *recent_manager;
+#endif
+
 #endif                          /* _COMMON_H */
