@@ -66,10 +66,10 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
     gchar *path4;
     GString *xml;
     gchar *xml_string;
-    gint count;
     GtkTreePath *treepath;
     gint source_id;
     gint bitrate;
+    gchar *buf = NULL;
 
     message_type = dbus_message_get_type(message);
     sender = dbus_message_get_sender(message);
@@ -100,7 +100,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
             // printf("Path matched %s\n", dbus_message_get_path(message));
             if (message_type == DBUS_MESSAGE_TYPE_SIGNAL) {
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "Open") == 0) {
-                    shutdown();
+                    mplayer_shutdown();
                     dbus_error_init(&error);
                     if (dbus_message_get_args
                         (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
@@ -109,35 +109,25 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                                 gtk_list_store_clear(playliststore);
                                 gtk_list_store_clear(nonrandomplayliststore);
                                 selection = NULL;
-                                playlist = detect_playlist(s);
-                                if (!playlist) {
-                                    add_item_to_playlist(s, playlist);
+                                if (!uri_exists(s) && !streaming_media(s)) {
+                                    buf = g_filename_to_uri(s, NULL, NULL);
                                 } else {
-                                    if (!parse_playlist(s)) {
-                                        add_item_to_playlist(s, playlist);
+                                    buf = g_strdup(s);
+                                }
+                                playlist = detect_playlist(buf);
+                                if (!playlist) {
+                                    add_item_to_playlist(buf, playlist);
+                                } else {
+                                    if (!parse_playlist(buf)) {
+                                        add_item_to_playlist(buf, playlist);
                                     }
                                 }
+                                g_free(buf);
                                 if (gtk_tree_model_get_iter_first
                                     (GTK_TREE_MODEL(playliststore), &iter)) {
-                                    gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter,
-                                                       ITEM_COLUMN, &s, COUNT_COLUMN, &count,
-                                                       PLAYLIST_COLUMN, &playlist, -1);
-                                    set_media_info_name(s);
-                                    play_file(s, playlist);
-                                    gtk_list_store_set(playliststore, &iter, COUNT_COLUMN,
-                                                       count + 1, -1);
+                                    play_iter(&iter);
                                 }
 
-                                if (GTK_IS_TREE_SELECTION(selection)) {
-                                    treepath =
-                                        gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore),
-                                                                &iter);
-                                    gtk_tree_selection_select_path(selection, treepath);
-                                    if (GTK_IS_WIDGET(list))
-                                        gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list), treepath,
-                                                                     NULL, FALSE, 0, 0);
-                                    gtk_tree_path_free(treepath);
-                                }
                             }
                         }
 
@@ -148,23 +138,24 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                 }
 
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "OpenPlaylist") == 0) {
-                    shutdown();
+                    mplayer_shutdown();
                     dbus_error_init(&error);
                     if (dbus_message_get_args
                         (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
                         gtk_list_store_clear(playliststore);
                         gtk_list_store_clear(nonrandomplayliststore);
                         selection = NULL;
-                        if (!parse_playlist(s)) {
-                            add_item_to_playlist(s, 1);
+                        if (!uri_exists(s) && !streaming_media(s)) {
+                            buf = g_filename_to_uri(s, NULL, NULL);
+                        } else {
+                            buf = g_strdup(s);
                         }
+                        if (!parse_playlist(buf)) {
+                            add_item_to_playlist(buf, 1);
+                        }
+                        g_free(buf);
                         if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter)) {
-                            gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN,
-                                               &s, COUNT_COLUMN, &count, PLAYLIST_COLUMN, &playlist,
-                                               -1);
-                            set_media_info_name(s);
-                            play_file(s, playlist);
-                            gtk_list_store_set(playliststore, &iter, COUNT_COLUMN, count + 1, -1);
+                            play_iter(&iter);
                         }
 
                         if (GTK_IS_TREE_SELECTION(selection)) {
@@ -176,6 +167,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                                                              FALSE, 0, 0);
                             gtk_tree_path_free(treepath);
                         }
+                        g_idle_add(set_update_gui, NULL);
 
                     } else {
                         dbus_error_free(&error);
@@ -184,7 +176,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                 }
 
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "OpenButton") == 0) {
-                    shutdown();
+                    mplayer_shutdown();
                     dbus_error_init(&error);
                     if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &s,
                                               DBUS_TYPE_STRING, &hrefid, DBUS_TYPE_INVALID)) {
@@ -199,8 +191,38 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     dbus_error_init(&error);
                     if (dbus_message_get_args
                         (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
-                        if (strlen(s) > 0)
-                            add_item_to_playlist(s, 0);
+                        if (strlen(s) > 0) {
+                            if (!uri_exists(s) && !streaming_media(s)) {
+                                buf = g_filename_to_uri(s, NULL, NULL);
+                            } else {
+                                buf = g_strdup(s);
+                            }
+                            playlist = detect_playlist(buf);
+                            if (!playlist) {
+                                add_item_to_playlist(buf, playlist);
+                            } else {
+                                if (!parse_playlist(buf)) {
+                                    add_item_to_playlist(buf, playlist);
+                                }
+                            }
+                            g_free(buf);
+                            g_idle_add(set_update_gui, NULL);
+                            // if play on add is set, play the last item on the playlist
+                        }
+                    } else {
+                        dbus_error_free(&error);
+                    }
+                    return DBUS_HANDLER_RESULT_HANDLED;
+                }
+
+                if (g_ascii_strcasecmp(dbus_message_get_member(message), "SetPlaylistName") == 0) {
+                    dbus_error_init(&error);
+                    if (dbus_message_get_args
+                        (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
+                        if (playlistname != NULL)
+                            g_free(playlistname);
+                        playlistname = g_strdup(s);
+                        g_idle_add(set_update_gui, NULL);
                     } else {
                         dbus_error_free(&error);
                     }
@@ -208,12 +230,12 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                 }
 
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "Close") == 0) {
-                    shutdown();
+                    mplayer_shutdown();
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
                 if (g_ascii_strcasecmp(dbus_message_get_member(message), "Quit") == 0) {
-                    shutdown();
+                    mplayer_shutdown();
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
@@ -445,6 +467,11 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                         (message, &error, DBUS_TYPE_DOUBLE, &(idledata->cachepercent),
                          DBUS_TYPE_INVALID)) {
                         g_idle_add(set_progress_value, idledata);
+                        if (idledata->cachepercent > 0.99 && idledata->retry_on_full_cache) {
+                            if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter)) {
+                                play_iter(&iter);
+                            }
+                        }
                     } else {
                         dbus_error_free(&error);
                     }
@@ -457,7 +484,8 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     if (dbus_message_get_args
                         (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
                         g_strlcpy(idledata->progress_text, s, 1024);
-                        g_idle_add(set_progress_text, idledata);
+                        if (state != PLAYING)
+                            g_idle_add(set_progress_text, idledata);
                     } else {
                         dbus_error_free(&error);
                     }
@@ -719,15 +747,16 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                 }
                 if (dbus_message_is_method_call(message, "com.gnome.mplayer", "GetFullScreen")) {
                     reply_message = dbus_message_new_method_return(message);
-                    dbus_message_append_args(reply_message, DBUS_TYPE_BOOLEAN, fullscreen,
+                    dbus_message_append_args(reply_message, DBUS_TYPE_BOOLEAN, &fullscreen,
                                              DBUS_TYPE_INVALID);
                     dbus_connection_send(connection, reply_message, NULL);
                     dbus_message_unref(reply_message);
                     return DBUS_HANDLER_RESULT_HANDLED;
                 }
                 if (dbus_message_is_method_call(message, "com.gnome.mplayer", "GetShowControls")) {
+                    showcontrols = get_show_controls();
                     reply_message = dbus_message_new_method_return(message);
-                    dbus_message_append_args(reply_message, DBUS_TYPE_BOOLEAN, fullscreen,
+                    dbus_message_append_args(reply_message, DBUS_TYPE_BOOLEAN, &showcontrols,
                                              DBUS_TYPE_INVALID);
                     dbus_connection_send(connection, reply_message, NULL);
                     dbus_message_unref(reply_message);
@@ -837,6 +866,46 @@ void dbus_open_next()
     }
 }
 
+void dbus_open(gchar * arg)
+{
+    gchar *path;
+    gchar *localarg;
+    DBusMessage *message;
+
+    if (connection != NULL) {
+        path = g_strdup_printf("/");
+        if (replace_and_play) {
+            message = dbus_message_new_signal(path, "com.gnome.mplayer", "Open");
+        } else {
+            message = dbus_message_new_signal(path, "com.gnome.mplayer", "Add");
+        }
+        localarg = g_strdup(arg);
+        dbus_message_append_args(message, DBUS_TYPE_STRING, &localarg, DBUS_TYPE_INVALID);
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+        g_free(localarg);
+        g_free(path);
+    }
+}
+
+void dbus_set_playlist_name(gchar * arg)
+{
+    gchar *path;
+    gchar *localarg;
+    DBusMessage *message;
+
+    if (connection != NULL) {
+        path = g_strdup_printf("/");
+        message = dbus_message_new_signal(path, "com.gnome.mplayer", "SetPlaylistName");
+        localarg = g_strdup(arg);
+        dbus_message_append_args(message, DBUS_TYPE_STRING, &localarg, DBUS_TYPE_INVALID);
+        dbus_connection_send(connection, message, NULL);
+        dbus_message_unref(message);
+        g_free(localarg);
+        g_free(path);
+    }
+}
+
 void dbus_cancel()
 {
     gchar *path;
@@ -1010,6 +1079,8 @@ gboolean dbus_hookup(gint windowid, gint controlid)
     DBusMessage *reply_message;
     gint id;
     gint ret;
+    gchar *filename;
+    gboolean local_rnp;
 
     dbus_error_init(&error);
     connection = dbus_bus_get_private(type, &error);
@@ -1040,13 +1111,14 @@ gboolean dbus_hookup(gint windowid, gint controlid)
         printf("Using match: %s\n", match);
     g_free(match);
     dbus_error_free(&error);
-
-    match = g_strdup_printf("type='signal',interface='org.gnome.SettingsDaemon.MediaKeys'");
-    dbus_bus_add_match(connection, match, &error);
-    if (verbose)
-        printf("Using match: %s\n", match);
-    g_free(match);
-    dbus_error_free(&error);
+    if (use_mediakeys) {
+        match = g_strdup_printf("type='signal',interface='org.gnome.SettingsDaemon.MediaKeys'");
+        dbus_bus_add_match(connection, match, &error);
+        if (verbose)
+            printf("Using match: %s\n", match);
+        g_free(match);
+        dbus_error_free(&error);
+    }
 
     dbus_connection_add_filter(connection, filter_func, NULL, NULL);
 
@@ -1057,8 +1129,6 @@ gboolean dbus_hookup(gint windowid, gint controlid)
         path = g_strdup_printf("com.gnome.mplayer.cid%i", control_id);
         ret = dbus_bus_request_name(connection, path, 0, NULL);
         g_free(path);
-        if (single_instance && ret > 1)
-            exit(1);
         path = g_strdup_printf("/control/%i", control_id);
         id = control_id;
         reply_message = dbus_message_new_signal(path, "com.gecko.mediaplayer", "Ready");
@@ -1068,8 +1138,23 @@ gboolean dbus_hookup(gint windowid, gint controlid)
         g_free(path);
     } else {
         ret = dbus_bus_request_name(connection, "com.gnome.mplayer", 0, NULL);
-        if (single_instance && ret > 1)
+        if (single_instance && ret > 1) {
+            if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter)) {
+                gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN, &filename,
+                                   -1);
+                dbus_open(filename);
+                local_rnp = replace_and_play;
+                while (gtk_tree_model_iter_next(GTK_TREE_MODEL(playliststore), &iter)) {
+                    replace_and_play = FALSE;
+                    gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN, &filename,
+                                       -1);
+                    dbus_open(filename);
+                }
+                if (playlistname != NULL && local_rnp)
+                    dbus_set_playlist_name(playlistname);
+            }
             exit(1);
+        }
     }
 
 
