@@ -34,11 +34,22 @@ void update_gui()
         gtk_widget_hide(prev_event_box);
         gtk_widget_hide(next_event_box);
         gtk_widget_set_sensitive(GTK_WIDGET(menuitem_edit_random), FALSE);
+        gtk_widget_hide(GTK_WIDGET(menuitem_prev));
+        gtk_widget_hide(GTK_WIDGET(menuitem_next));
     } else {
         gtk_widget_show(prev_event_box);
         gtk_widget_show(next_event_box);
         gtk_widget_set_sensitive(GTK_WIDGET(menuitem_edit_random), TRUE);
+        gtk_widget_show(GTK_WIDGET(menuitem_prev));
+        gtk_widget_show(GTK_WIDGET(menuitem_next));
     }
+
+    if (gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL) > 0) {
+        gtk_widget_set_sensitive(GTK_WIDGET(menuitem_edit_loop), TRUE);
+    } else {
+        gtk_widget_set_sensitive(GTK_WIDGET(menuitem_edit_loop), FALSE);
+    }
+
 }
 
 gboolean playlist_popup_handler(GtkWidget * widget, GdkEvent * event, void *data)
@@ -68,13 +79,16 @@ gboolean button_release_callback(GtkWidget * widget, GdkEvent * event, gpointer
     // GtkTreeSelection *sel;
     gint pos;
     GdkEventButton *event_button;
+    gchar *buf;
 
     event_button = (GdkEventButton *) event;
 
     gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(list), event_button->x, event_button->y, &path,
                                   NULL, NULL, NULL);
     if (path != NULL) {
-        pos = (gint) g_strtod(gtk_tree_path_to_string(path), NULL);
+        buf = gtk_tree_path_to_string(path);
+        pos = (gint) g_strtod(buf, NULL);
+        g_free(buf);
         gtk_tree_path_free(path);
 
         if (pos == 0) {
@@ -123,7 +137,8 @@ void playlist_set_subtitle_callback(GtkMenuItem * menuitem, void *data)
                                              GTK_FILE_CHOOSER_ACTION_OPEN,
                                              GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                              GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
-        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), path);
+        gtk_widget_show(dialog);
+        gtk_file_chooser_set_current_folder_uri(GTK_FILE_CHOOSER(dialog), path);
         if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
             subtitle = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
             gtk_list_store_set(playliststore, &localiter, SUBTITLE_COLUMN, subtitle, -1);
@@ -139,8 +154,6 @@ gboolean playlist_drop_callback(GtkWidget * widget, GdkDragContext * dc,
                                 gint x, gint y, GtkSelectionData * selection_data,
                                 guint info, guint t, gpointer data)
 {
-    gchar *filename;
-    GtkTreeIter localiter;
     gchar **list;
     gint i = 0;
     gint playlist;
@@ -154,17 +167,14 @@ gboolean playlist_drop_callback(GtkWidget * widget, GdkDragContext * dc,
         return FALSE;
 
     if ((info == DRAG_INFO_0) || (info == DRAG_INFO_1) || (info == DRAG_INFO_2)) {
-        filename = g_filename_from_uri((const gchar *) selection_data->data, NULL, NULL);
-        if (filename == NULL)
-            return FALSE;
-        list = g_strsplit(filename, "\n", 0);
+        list = g_uri_list_extract_uris((const gchar *) selection_data->data);
 
         while (list[i] != NULL) {
-            g_strchomp(list[i]);
             if (strlen(list[i]) > 0) {
-                printf("filename = '%s'\n", list[i]);
-                if (g_file_test(list[i], G_FILE_TEST_IS_DIR)) {
+                if (is_uri_dir(list[i])) {
+                    create_folder_progress_window();
                     add_folder_to_playlist_callback(list[i], NULL);
+                    destroy_folder_progress_window();
                 } else {
                     playlist = detect_playlist(list[i]);
 
@@ -172,7 +182,7 @@ gboolean playlist_drop_callback(GtkWidget * widget, GdkDragContext * dc,
                         add_item_to_playlist(list[i], playlist);
                     } else {
                         if (!parse_playlist(list[i])) {
-                            localiter = add_item_to_playlist(list[i], playlist);
+                            add_item_to_playlist(list[i], playlist);
                         }
                     }
                 }
@@ -180,18 +190,20 @@ gboolean playlist_drop_callback(GtkWidget * widget, GdkDragContext * dc,
             i++;
         }
 
-        if (gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL) < 2) {
+        if (gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL) == 1) {
             gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter);
-            gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN, &filename,
-                               PLAYLIST_COLUMN, &playlist, -1);
-
-            shutdown();
-            play_file(filename, playlist);
-            g_free(filename);
+            play_iter(&iter);
 
         } else {
             gtk_widget_set_sensitive(GTK_WIDGET(menuitem_edit_random), TRUE);
         }
+
+        if (gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL) > 0) {
+            gtk_widget_set_sensitive(GTK_WIDGET(menuitem_edit_loop), TRUE);
+        } else {
+            gtk_widget_set_sensitive(GTK_WIDGET(menuitem_edit_loop), FALSE);
+        }
+
         g_strfreev(list);
     }
     update_gui();
@@ -233,10 +245,9 @@ gboolean playlist_motion_callback(GtkWidget * widget, GdkEventMotion * event, gp
 void save_playlist(GtkWidget * widget, void *data)
 {
     GtkWidget *dialog;
-    gchar *filename;
-    gchar *new_filename;
+    gchar *uri;
+    gchar *new_uri;
     GtkFileFilter *filter;
-    GConfClient *gconf;
     gchar *last_dir;
 
     dialog = gtk_file_chooser_dialog_new(_("Save Playlist"),
@@ -244,10 +255,15 @@ void save_playlist(GtkWidget * widget, void *data)
                                          GTK_FILE_CHOOSER_ACTION_SAVE,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
-    gconf = gconf_client_get_default();
-    last_dir = gconf_client_get_string(gconf, LAST_DIR, NULL);
+
+#ifdef GIO_ENABLED
+    gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog), FALSE);
+#endif
+    gtk_widget_show(dialog);
+    init_preference_store();
+    last_dir = read_preference_string(LAST_DIR);
     if (last_dir != NULL)
-        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), last_dir);
+        gtk_file_chooser_set_current_folder_uri(GTK_FILE_CHOOSER(dialog), last_dir);
 
     gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), TRUE);
     filter = gtk_file_filter_new();
@@ -262,26 +278,26 @@ void save_playlist(GtkWidget * widget, void *data)
 
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
 
-        filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
-        last_dir = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(dialog));
-        gconf_client_set_string(gconf, LAST_DIR, last_dir, NULL);
+        uri = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog));
+        last_dir = gtk_file_chooser_get_current_folder_uri(GTK_FILE_CHOOSER(dialog));
+        write_preference_string(LAST_DIR, last_dir);
         g_free(last_dir);
 
-        if (g_strrstr(filename, ".m3u") != NULL) {
-            save_playlist_m3u(filename);
+        if (g_strrstr(uri, ".m3u") != NULL) {
+            save_playlist_m3u(uri);
         }
-        if (g_strrstr(filename, ".pls") != NULL) {
-            save_playlist_pls(filename);
+        if (g_strrstr(uri, ".pls") != NULL) {
+            save_playlist_pls(uri);
         }
 
-        if (g_strrstr(filename, ".") == NULL) {
-            new_filename = g_strdup_printf("%s.pls", filename);
-            save_playlist_pls(new_filename);
-            g_free(new_filename);
+        if (g_strrstr(uri, ".") == NULL) {
+            new_uri = g_strdup_printf("%s.pls", uri);
+            save_playlist_pls(new_uri);
+            g_free(new_uri);
         }
-        g_free(filename);
+        g_free(uri);
     }
-
+    release_preference_store();
     gtk_widget_destroy(dialog);
 
 }
@@ -291,7 +307,6 @@ void load_playlist(GtkWidget * widget, void *data)
     GtkWidget *dialog;
     gchar *filename;
     GtkFileFilter *filter;
-    GConfClient *gconf;
     gchar *last_dir;
 
     dialog = gtk_file_chooser_dialog_new(_("Open Playlist"),
@@ -299,11 +314,11 @@ void load_playlist(GtkWidget * widget, void *data)
                                          GTK_FILE_CHOOSER_ACTION_OPEN,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
-
-    gconf = gconf_client_get_default();
-    last_dir = gconf_client_get_string(gconf, LAST_DIR, NULL);
+    gtk_widget_show(dialog);
+    init_preference_store();
+    last_dir = read_preference_string(LAST_DIR);
     if (last_dir != NULL)
-        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), last_dir);
+        gtk_file_chooser_set_current_folder_uri(GTK_FILE_CHOOSER(dialog), last_dir);
 
     filter = gtk_file_filter_new();
     gtk_file_filter_set_name(filter, _("Playlist (*.pls)"));
@@ -322,19 +337,21 @@ void load_playlist(GtkWidget * widget, void *data)
 
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
 
-        filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
-        last_dir = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(dialog));
-        gconf_client_set_string(gconf, LAST_DIR, last_dir, NULL);
+        filename = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog));
+        last_dir = gtk_file_chooser_get_current_folder_uri(GTK_FILE_CHOOSER(dialog));
+        write_preference_string(LAST_DIR, last_dir);
         g_free(last_dir);
 
         gtk_list_store_clear(playliststore);
         gtk_list_store_clear(nonrandomplayliststore);
 
+        create_folder_progress_window();
         if (!parse_playlist(filename)) {
             add_item_to_playlist(filename, 1);
         }
-
+        destroy_folder_progress_window();
     }
+    release_preference_store();
     update_gui();
     gtk_widget_destroy(dialog);
 }
@@ -352,6 +369,8 @@ void add_item_to_playlist_callback(gpointer data, gpointer user_data)
             add_item_to_playlist(filename, playlist);
         }
     }
+    set_item_add_info(filename);
+
     g_free(filename);
 }
 
@@ -362,23 +381,66 @@ gint compar(gpointer a, gpointer b)
 
 void add_folder_to_playlist_callback(gpointer data, gpointer user_data)
 {
-    gchar *filename = (gchar *) data;
+
+#ifdef GIO_ENABLED
+    gchar *uri = (gchar *) data;
+    GFile *file;
+    GFileEnumerator *dir;
+    GFileInfo *info;
+    GError *error;
+    gchar *sub_uri;
+    GSList *list = NULL;
+
+    error = NULL;
+    file = g_file_new_for_uri(uri);
+    dir = g_file_enumerate_children(file, "standard::*", G_FILE_QUERY_INFO_NONE, NULL, &error);
+    if (error != NULL)
+        printf("error message = %s\n", error->message);
+
+    if (dir != NULL) {
+        info = g_file_enumerator_next_file(dir, NULL, NULL);
+        while (info != NULL) {
+            if (g_file_info_get_file_type(info) & G_FILE_TYPE_DIRECTORY) {
+                sub_uri = g_strdup_printf("%s/%s", uri, g_file_info_get_name(info));
+                add_folder_to_playlist_callback((gpointer) sub_uri, NULL);
+                g_free(sub_uri);
+            } else {
+                sub_uri = g_strdup_printf("%s/%s", uri, g_file_info_get_name(info));
+                list = g_slist_prepend(list, sub_uri);
+                filecount++;
+            }
+            g_object_unref(info);
+            info = g_file_enumerator_next_file(dir, NULL, NULL);
+        }
+        list = g_slist_sort(list, (GCompareFunc) compar);
+        g_slist_foreach(list, &add_item_to_playlist_callback, NULL);
+        g_slist_free(list);
+        g_object_unref(dir);
+    }
+    g_object_unref(file);
+#else
+    gchar *uri = (gchar *) data;
+    gchar *filename;
     GDir *dir;
     gchar *name;
     gchar *subdir = NULL;
+    gchar *subdir_uri = NULL;
     GSList *list = NULL;
 
+    filename = g_filename_from_uri(uri, NULL, NULL);
     dir = g_dir_open(filename, 0, NULL);
     if (dir != NULL) {
         do {
             name = g_strdup(g_dir_read_name(dir));
             if (name != NULL) {
                 subdir = g_strdup_printf("%s/%s", filename, name);
+                subdir_uri = g_filename_to_uri(subdir, NULL, NULL);
                 if (g_file_test(subdir, G_FILE_TEST_IS_DIR)) {
-                    add_folder_to_playlist_callback((gpointer) subdir, NULL);
+                    add_folder_to_playlist_callback((gpointer) subdir_uri, NULL);
                     g_free(subdir);
+                    g_free(subdir_uri);
                 } else {
-                    list = g_slist_prepend(list, subdir);
+                    list = g_slist_prepend(list, subdir_uri);
                     filecount++;
                 }
                 g_free(name);
@@ -391,15 +453,14 @@ void add_folder_to_playlist_callback(gpointer data, gpointer user_data)
         g_slist_free(list);
         g_dir_close(dir);
     }
-
+#endif
 }
 
 void add_to_playlist(GtkWidget * widget, void *data)
 {
 
     GtkWidget *dialog;
-    GSList *filename;
-    GConfClient *gconf;
+    GSList *uris;
     gchar *last_dir;
     GtkTreeViewColumn *column;
     gchar *coltitle;
@@ -412,31 +473,39 @@ void add_to_playlist(GtkWidget * widget, void *data)
                                          GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 
     /*allow multiple files to be selected */
+    gtk_widget_show(dialog);
     gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);
+#ifdef GIO_ENABLED
+    gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog), FALSE);
+#endif
 
-    gconf = gconf_client_get_default();
-    last_dir = gconf_client_get_string(gconf, LAST_DIR, NULL);
+    init_preference_store();
+    last_dir = read_preference_string(LAST_DIR);
     if (last_dir != NULL) {
-        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), last_dir);
+        gtk_file_chooser_set_current_folder_uri(GTK_FILE_CHOOSER(dialog), last_dir);
         g_free(last_dir);
     }
 
+    count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL);
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
 
-        filename = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dialog));
+        uris = gtk_file_chooser_get_uris(GTK_FILE_CHOOSER(dialog));
 
-        last_dir = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(dialog));
-        gconf_client_set_string(gconf, LAST_DIR, last_dir, NULL);
+        last_dir = gtk_file_chooser_get_current_folder_uri(GTK_FILE_CHOOSER(dialog));
+        write_preference_string(LAST_DIR, last_dir);
         g_free(last_dir);
 
-        g_slist_foreach(filename, &add_item_to_playlist_callback, NULL);
-        g_slist_free(filename);
+        g_slist_foreach(uris, &add_item_to_playlist_callback, NULL);
+        g_slist_free(uris);
     }
     update_gui();
-    g_object_unref(G_OBJECT(gconf));
+    release_preference_store();
     column = gtk_tree_view_get_column(GTK_TREE_VIEW(list), 0);
-    count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(playliststore), NULL);
-    coltitle = g_strdup_printf(ngettext("Item to Play", "Items to Play", count));
+    if (playlistname != NULL && strlen(playlistname) > 0 && count == 0) {
+        coltitle = g_strdup_printf(_("%s items"), playlistname);
+    } else {
+        coltitle = g_strdup_printf(ngettext("Item to Play", "Items to Play", count));
+    }
     gtk_tree_view_column_set_title(column, coltitle);
     g_free(coltitle);
 
@@ -447,8 +516,7 @@ void add_folder_to_playlist(GtkWidget * widget, void *data)
 {
 
     GtkWidget *dialog;
-    GSList *filename;
-    GConfClient *gconf;
+    GSList *uris;
     gchar *last_dir;
     gchar *message;
 
@@ -458,30 +526,36 @@ void add_folder_to_playlist(GtkWidget * widget, void *data)
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 
+    gtk_widget_show(dialog);
     /*allow multiple files to be selected */
     gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);
+#ifdef GIO_ENABLED
+    gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog), FALSE);
+#endif
 
-    gconf = gconf_client_get_default();
-    last_dir = gconf_client_get_string(gconf, LAST_DIR, NULL);
+    init_preference_store();
+    last_dir = read_preference_string(LAST_DIR);
     if (last_dir != NULL) {
-        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), last_dir);
+        gtk_file_chooser_set_current_folder_uri(GTK_FILE_CHOOSER(dialog), last_dir);
         g_free(last_dir);
     }
 
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
 
-        filename = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dialog));
+        uris = gtk_file_chooser_get_uris(GTK_FILE_CHOOSER(dialog));
 
-        last_dir = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(dialog));
-        gconf_client_set_string(gconf, LAST_DIR, last_dir, NULL);
+        last_dir = gtk_file_chooser_get_current_folder_uri(GTK_FILE_CHOOSER(dialog));
+        write_preference_string(LAST_DIR, last_dir);
         g_free(last_dir);
 
         filecount = 0;
-        g_slist_foreach(filename, &add_folder_to_playlist_callback, NULL);
-        g_slist_free(filename);
+        create_folder_progress_window();
+        g_slist_foreach(uris, &add_folder_to_playlist_callback, NULL);
+        destroy_folder_progress_window();
+        g_slist_free(uris);
     }
     update_gui();
-    g_object_unref(G_OBJECT(gconf));
+    release_preference_store();
     gtk_widget_destroy(dialog);
     message =
         g_markup_printf_escaped(ngettext("\n\tFound %i file\n", "\n\tFound %i files\n", filecount),
@@ -498,14 +572,29 @@ void remove_from_playlist(GtkWidget * widget, gpointer data)
     GtkTreeView *view = (GtkTreeView *) data;
     GtkTreeIter localiter;
     GtkTreePath *path;
+    gchar *localfilename;
+    gchar *removedfilename;
 
     sel = gtk_tree_view_get_selection(view);
 
     if (gtk_tree_selection_get_selected(sel, NULL, &iter)) {
+        gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN, &removedfilename, -1);
         localiter = iter;
         gtk_tree_model_iter_next(GTK_TREE_MODEL(playliststore), &localiter);
         if (gtk_list_store_remove(playliststore, &iter)) {
             iter = localiter;
+            gtk_tree_model_get_iter_first(GTK_TREE_MODEL(nonrandomplayliststore), &localiter);
+            do {
+                gtk_tree_model_get(GTK_TREE_MODEL(nonrandomplayliststore), &localiter, ITEM_COLUMN,
+                                   &localfilename, -1);
+                if (localfilename != NULL) {
+                    if (g_ascii_strcasecmp(removedfilename, localfilename) == 0) {
+                        gtk_list_store_remove(nonrandomplayliststore, &localiter);
+                    }
+                    g_free(localfilename);
+                }
+            } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(nonrandomplayliststore), &localiter));
+
             if (!gtk_list_store_iter_is_valid(playliststore, &iter)) {
                 gtk_tree_model_get_iter_first(GTK_TREE_MODEL(playliststore), &iter);
             }
@@ -517,6 +606,7 @@ void remove_from_playlist(GtkWidget * widget, gpointer data)
                 gtk_tree_path_free(path);
             }
         }
+        g_free(removedfilename);
     }
     update_gui();
 
@@ -526,10 +616,12 @@ void clear_playlist(GtkWidget * widget, void *data)
 {
 
     dontplaynext = TRUE;
-    shutdown();
+    mplayer_shutdown();
     gtk_list_store_clear(playliststore);
     gtk_list_store_clear(nonrandomplayliststore);
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem_edit_random), FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(menuitem_edit_random), FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(menuitem_edit_loop), FALSE);
 }
 
 void move_item_up(GtkWidget * widget, void *data)
@@ -538,12 +630,15 @@ void move_item_up(GtkWidget * widget, void *data)
     GtkTreeIter localiter, a, b;
     GtkTreePath *path;
     gint pos = 0;
+    gchar *buf;
 
     sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
     if (gtk_tree_selection_get_selected(sel, NULL, &localiter)) {
         path = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &localiter);
         if (path != NULL) {
-            pos = (gint) g_strtod(gtk_tree_path_to_string(path), NULL);
+            buf = gtk_tree_path_to_string(path);
+            pos = (gint) g_strtod(buf, NULL);
+            g_free(buf);
             gtk_tree_path_free(path);
 
             if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(playliststore), &a, NULL, pos)) {
@@ -574,12 +669,15 @@ void move_item_down(GtkWidget * widget, void *data)
     GtkTreeIter localiter, a, b;
     GtkTreePath *path;
     gint pos = 0;
+    gchar *buf;
 
     sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
     if (gtk_tree_selection_get_selected(sel, NULL, &localiter)) {
         path = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &localiter);
         if (path != NULL) {
-            pos = (gint) g_strtod(gtk_tree_path_to_string(path), NULL);
+            buf = gtk_tree_path_to_string(path);
+            pos = (gint) g_strtod(buf, NULL);
+            g_free(buf);
             gtk_tree_path_free(path);
             if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(playliststore), &a, NULL, pos)) {
                 if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(playliststore), &b, NULL, pos + 1)) {
@@ -608,35 +706,18 @@ gboolean playlist_select_callback(GtkTreeView * view, GtkTreePath * path,
                                   GtkTreeViewColumn * column, gpointer data)
 {
 
-    gchar *filename = NULL;
-    gint count;
-    gint playlist;
-    gchar *cmd = NULL;
-
     if (gtk_tree_model_get_iter(GTK_TREE_MODEL(playliststore), &iter, path)) {
-        gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, ITEM_COLUMN, &filename,
-                           COUNT_COLUMN, &count, PLAYLIST_COLUMN, &playlist, -1);
-
-        if (state == QUIT) {
-            play_file(filename, playlist);
-        } else {
-            cmd = g_strdup_printf("loadfile \"%s\"\n", filename);
-            send_command(cmd);
-            g_free(cmd);
-            if (state != PLAYING)
-                play_callback(NULL, NULL, NULL);
-        }
-        set_media_info_name(filename);
-        gtk_list_store_set(playliststore, &iter, COUNT_COLUMN, count + 1, -1);
-        g_free(filename);
-
+        dontplaynext = TRUE;
+        mplayer_shutdown();
+        play_iter(&iter);
+        dontplaynext = FALSE;
     }
     return FALSE;
 }
 
 void playlist_close(GtkWidget * widget, void *data)
 {
-    menuitem_view_playlist_callback(NULL, NULL);
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem_view_playlist), FALSE);
 }
 
 void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
@@ -665,10 +746,11 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
     gint count;
     GtkTargetEntry target_entry[3];
     gint i = 0;
+    gint handle_size;
 
     g_value_init(&value, G_TYPE_BOOLEAN);
 
-    if (GTK_IS_WIDGET(plvbox)) {
+    if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem_view_playlist)) == FALSE) {
         if (GTK_WIDGET_VISIBLE(plvbox)) {
             if (idledata->videopresent == FALSE) {
                 gdk_window_get_geometry(window->window, &x, &y, &window_width, &window_height,
@@ -677,24 +759,33 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
                 stored_window_height = -1;
                 gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
                 gtk_widget_hide(GTK_WIDGET(fixed));
-                gtk_widget_show_all(media_label);
+                gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem_view_info), TRUE);
                 gtk_widget_show(vbox);
                 gtk_widget_set_size_request(window, -1, -1);
             } else {
+                gtk_widget_style_get(pane, "handle-size", &handle_size, NULL);
                 gtk_window_set_resizable(GTK_WINDOW(window), TRUE);
                 gtk_widget_set_size_request(window, -1, -1);
                 gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, TRUE);
                 window_width = -1;
                 window_height = -1;
-                if (stored_window_width != -1)
-                    gtk_window_resize(GTK_WINDOW(window), stored_window_width,
-                                      stored_window_height);
+                gtk_window_get_size(GTK_WINDOW(window), &window_width, &window_height);
+                gtk_widget_hide(plvbox);
+                if (vertical_layout) {
+                    gtk_window_resize(GTK_WINDOW(window), window_width,
+                                      window_height - plvbox->allocation.height - handle_size);
+                } else {
+                    gtk_window_resize(GTK_WINDOW(window),
+                                      window_width - plvbox->allocation.width - handle_size,
+                                      window_height);
+                }
             }
             gtk_container_remove(GTK_CONTAINER(pane), plvbox);
             stored_window_width = -1;
             stored_window_height = -1;
             plvbox = NULL;
             selection = NULL;
+            playlist_visible = FALSE;
         }
 
     } else {
@@ -702,13 +793,25 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
         if (idledata->videopresent == FALSE) {
             if (window_width != -1)
                 gtk_window_resize(GTK_WINDOW(window), window_width, window_height);
-            gtk_widget_hide(vbox);
+            if (vertical_layout) {
+                gtk_widget_hide(fixed);
+            } else {
+                gtk_widget_hide(vbox);
+            }
         } else {
-            if (!restore_playlist) {
+            // set the window size properly when coming out of fullscreen mode.
+            if (restore_playlist) {
+                if (vertical_layout) {
+                    stored_window_height = restore_pane;
+                } else {
+                    stored_window_width = restore_pane;
+                }
+				restore_playlist = FALSE;
+            } else {
                 gdk_window_get_geometry(window->window, &x, &y, &stored_window_width,
                                         &stored_window_height, &depth);
             }
-        }
+		}
         plvbox = gtk_vbox_new(FALSE, 12);
         hbox = gtk_hbox_new(FALSE, 12);
         gtk_box_set_homogeneous(GTK_BOX(hbox), FALSE);
@@ -770,6 +873,7 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
         }
         gtk_tree_view_column_set_expand(column, TRUE);
         //gtk_tree_view_column_set_max_width(column, 40);
+        g_object_set(renderer, "width-chars", 40, NULL);
         gtk_tree_view_column_set_resizable(column, TRUE);
         gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
 
@@ -777,7 +881,19 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
         column = gtk_tree_view_column_new_with_attributes(_("Artist"),
                                                           renderer, "text", ARTIST_COLUMN, NULL);
         gtk_tree_view_column_set_expand(column, TRUE);
-        gtk_tree_view_column_set_max_width(column, 20);
+        //gtk_tree_view_column_set_max_width(column, 20);
+        g_object_set(renderer, "width-chars", 20, NULL);
+        gtk_tree_view_column_set_alignment(column, 0.0);
+        gtk_tree_view_column_set_resizable(column, TRUE);
+        gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
+
+        renderer = gtk_cell_renderer_text_new();
+        column = gtk_tree_view_column_new_with_attributes(_("Album"),
+                                                          renderer, "text", ALBUM_COLUMN, NULL);
+        gtk_tree_view_column_set_expand(column, TRUE);
+        //gtk_tree_view_column_set_max_width(column, 20);
+        g_object_set(renderer, "width-chars", 20, NULL);
+        gtk_tree_view_column_set_alignment(column, 0.0);
         gtk_tree_view_column_set_resizable(column, TRUE);
         gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
 
@@ -787,6 +903,7 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
                                                           renderer, "text", LENGTH_COLUMN, NULL);
         //gtk_tree_view_column_set_expand(column, FALSE);
         gtk_tree_view_column_set_alignment(column, 1.0);
+        g_object_set(renderer, "xalign", 1.0, NULL);
         gtk_tree_view_column_set_resizable(column, FALSE);
         gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
 
@@ -794,6 +911,7 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
         renderer = gtk_cell_renderer_text_new();
         column = gtk_tree_view_column_new_with_attributes("", renderer, "text", COUNT_COLUMN, NULL);
         //gtk_tree_view_column_set_expand(column, FALSE);
+        g_object_set(renderer, "xalign", 1.0, NULL);
         gtk_tree_view_column_set_resizable(column, FALSE);
         gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
 
@@ -896,18 +1014,21 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
         gtk_box_pack_start(GTK_BOX(plvbox), hbox, FALSE, FALSE, 0);
 
 
-        gtk_paned_pack2(GTK_PANED(pane), plvbox, FALSE, FALSE);
+        gtk_paned_pack2(GTK_PANED(pane), plvbox, TRUE, TRUE);
+        adjust_paned_rules();
 
+        gtk_widget_style_get(pane, "handle-size", &handle_size, NULL);
         if (vertical_layout) {
             gtk_widget_set_size_request(plvbox, -1, 150);
             if (idledata->videopresent)
                 gtk_window_resize(GTK_WINDOW(window), stored_window_width,
-                                  stored_window_height + 150);
+                                  stored_window_height + 150 + handle_size);
         } else {
             gtk_widget_set_size_request(plvbox, 300, -1);
-            if (idledata->videopresent)
-                gtk_window_resize(GTK_WINDOW(window), stored_window_width + 300,
+            if (idledata->videopresent) {
+                gtk_window_resize(GTK_WINDOW(window), stored_window_width + 300 + handle_size,
                                   stored_window_height);
+            }
         }
         gtk_widget_show_all(plvbox);
         if (idledata->videopresent == FALSE) {
@@ -927,7 +1048,7 @@ void menuitem_view_playlist_callback(GtkMenuItem * menuitem, void *data)
                                  "button_press_event",
                                  G_CALLBACK(playlist_popup_handler), G_OBJECT(playlist_popup_menu));
         gtk_widget_show_all(GTK_WIDGET(playlist_popup_menu));
-
+        playlist_visible = TRUE;
 
         gtk_widget_grab_default(close);
     }
