@@ -28,19 +28,44 @@
 #include "gui.h"
 #include "common.h"
 #include <string.h>
-#include <stdlib.h>
 #include <unistd.h>
 
 #ifdef HAVE_ASOUNDLIB
 #include <asoundlib.h>
 #endif
 
+#ifdef HAVE_GCONF
+#include <gconf/gconf.h>
+#include <gconf/gconf-client.h>
+#include <gconf/gconf-value.h>
+GConfClient *gconf;
+#else
+GKeyFile *config;
+#endif
+
+#ifdef GIO_ENABLED
+#include <gio/gio.h>
+#endif
+
+#ifdef HAVE_GPOD
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+#include <mntent_compat.h>
+#include <gpod/itdb.h>
+#endif
+
+#ifdef HAVE_MUSICBRAINZ
+#include <musicbrainz3/mb_c.h>
+#include <curl/curl.h>
+#endif
+
 gint get_bitrate(gchar * name);
 void strip_unicode(gchar * data, gsize len);
-gint play_file(gchar * filename, gint playlist);
+gint play_iter(GtkTreeIter * playiter);
 gint detect_playlist(gchar * filename);
 gchar *metadata_to_utf8(gchar * string);
-gint parse_playlist(gchar * filename);
+gint parse_playlist(gchar * uri);
 gint parse_basic(gchar * filename);
 gint parse_ram(gchar * filename);
 gint parse_cdda(gchar * filename);
@@ -49,12 +74,39 @@ gboolean save_playlist_pls(gchar * filename);
 gboolean save_playlist_m3u(gchar * filename);
 gchar *get_path(gchar * filename);
 gboolean update_mplayer_config();
-gboolean send_command(gchar * command);
+gboolean send_command(gchar * command, gboolean retain_pause);
 gboolean streaming_media(gchar * filename);
 gboolean device_name(gchar * filename);
-GtkTreeIter add_item_to_playlist(gchar * itemname, gint playlist);
+gboolean add_item_to_playlist(gchar * itemname, gint playlist);
 gboolean next_item_in_playlist(GtkTreeIter * iter);
 void copy_playlist(GtkListStore * source, GtkListStore * dest);
 void randomize_playlist(GtkListStore * store);
 gdouble get_alsa_volume();
+gchar *seconds_to_string(gfloat seconds);
+
+void init_preference_store();
+gboolean read_preference_bool(gchar * key);
+gint read_preference_int(gchar * key);
+gfloat read_preference_float(gchar * key);
+gchar *read_preference_string(gchar * key);
+void write_preference_bool(gchar * key, gboolean value);
+void write_preference_int(gchar * key, gint value);
+void write_preference_float(gchar * key, gfloat value);
+void write_preference_string(gchar * key, gchar * value);
+void release_preference_store();
+
+gchar *get_localfile_from_uri(gchar * uri);
+gboolean is_uri_dir(gchar * uri);
+gboolean uri_exists(gchar * uri);
+
+#ifdef HAVE_GPOD
+gchar *find_gpod_mount_point();
+gboolean gpod_load_tracks(gchar * mount_point);
+#endif
+
+gchar *get_cover_art_url(gchar * artist, gchar * title, gchar * album);
+gpointer get_cover_art(gpointer data);
+
+gchar *gmp_tempname(gchar * path, const gchar * name_template);
+
 #endif                          // _SUPPORT_H
