@@ -25,53 +25,52 @@
 #include "thread.h"
 #include "common.h"
 
-void shutdown()
+void mplayer_shutdown()
 {
     // printf("state = %i quit = %i\n",state,QUIT);
     if (state != QUIT) {
         idledata->percent = 1.0;
         g_idle_add(set_progress_value, idledata);
         g_idle_add(set_stop, idledata);
-        send_command("quit\n");
+        send_command("quit\n", FALSE);
     }
 
 }
 
-gboolean send_command(gchar * command)
+gboolean send_command(gchar * command, gboolean retain_pause)
 {
-    gint ret;
+    gchar *cmd;
+    GIOStatus result;
+
+    if (retain_pause) {
+        if (use_pausing_keep_force) {
+            cmd = g_strdup_printf("pausing_keep_force %s", command);
+        } else {
+            cmd = g_strdup_printf("pausing_keep %s", command);
+        }
+    } else {
+        cmd = g_strdup(command);
+    }
 
     if (verbose > 1)
-        printf("send command = %s\n", command);
-    ret = write(std_in, command, strlen(command));
-    fsync(std_in);
-    if (ret < 0) {
-        return FALSE;
-    } else {
-        return TRUE;
+        printf("send command = %s\n", cmd);
+
+    if (channel_in) {
+        result = g_io_channel_write_chars(channel_in, cmd, -1, NULL, NULL);
+        result = g_io_channel_flush(channel_in, NULL);
     }
 
+    g_free(cmd);
+    return TRUE;
+
 }
 
 gboolean play(void *data)
 {
     PlayData *p = (PlayData *) data;
-    GtkTreePath *path;
-    gint count;
 
     if (ok_to_play && p != NULL) {
-        play_file(p->filename, p->playlist);
-        if (gtk_list_store_iter_is_valid(playliststore, &iter)) {
-            gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, COUNT_COLUMN, &count, -1);
-            gtk_list_store_set(playliststore, &iter, COUNT_COLUMN, count + 1, -1);
-            if (GTK_IS_TREE_SELECTION(selection)) {
-                path = gtk_tree_model_get_path(GTK_TREE_MODEL(playliststore), &iter);
-                gtk_tree_selection_select_path(selection, path);
-                if (GTK_IS_WIDGET(list))
-                    gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(list), path, NULL, FALSE, 0, 0);
-                gtk_tree_path_free(path);
-            }
-        }
+        play_iter(&iter);
     }
     g_free(p);
 
@@ -80,11 +79,16 @@ gboolean play(void *data)
 
 gboolean thread_complete(GIOChannel * source, GIOCondition condition, gpointer data)
 {
+    if (verbose > 1)
+        printf("thread complete\n");
     g_idle_add(set_stop, idledata);
     state = QUIT;
     g_source_remove(watch_in_id);
     g_source_remove(watch_err_id);
-    g_mutex_unlock(thread_running);
+    if (verbose > 1)
+        printf("thread_complete, unlocking thread running\n");
+    // g_mutex_unlock(thread_running);
+	g_cond_signal(mplayer_complete_cond);
     return FALSE;
 }
 
@@ -101,24 +105,31 @@ gboolean thread_reader_error(GIOChannel * source, GIOCondition condition, gpoint
         return FALSE;
     }
 
+    mplayer_output = g_string_new("");
+    status = g_io_channel_read_line_string(source, mplayer_output, NULL, NULL);
 
     if (state == QUIT) {
-        if (verbose > 1)
+        if (verbose > 1) {
             printf("Thread Error: state = QUIT, shutting down\n");
+            printf("ERROR: %s", mplayer_output->str);
+        }
+        g_string_free(mplayer_output, TRUE);
         g_idle_add(set_stop, idledata);
         state = QUIT;
         g_source_remove(watch_in_id);
         g_source_remove(watch_in_hup_id);
-        g_mutex_unlock(thread_running);
+        //g_mutex_unlock(thread_running);
+		g_cond_signal(mplayer_complete_cond);
         return FALSE;
     }
 
-    mplayer_output = g_string_new("");
-
-    status = g_io_channel_read_line_string(source, mplayer_output, NULL, NULL);
-    if (verbose && strstr(mplayer_output->str, "ANS_") == NULL)
+    if (verbose == 1 && strstr(mplayer_output->str, "ANS_") == NULL)
         printf("ERROR: %s", mplayer_output->str);
 
+    // print out everything in really verbose mode
+    if (verbose > 1)
+        printf("thread reader error: %s", mplayer_output->str);
+
     if (strstr(mplayer_output->str, "Couldn't open DVD device") != 0) {
         error_msg = g_strdup(mplayer_output->str);
     }
@@ -132,9 +143,11 @@ gboolean thread_reader_error(GIOChannel * source, GIOCondition condition, gpoint
             strstr(mplayer_output->str, "/dev/rtc") == NULL &&
             strstr(mplayer_output->str, "registry file") == NULL) {
             // error_msg = g_strdup(mplayer_output->str);
-            error_msg =
-                g_strdup_printf(_("Failed to open %s"),
-                                mplayer_output->str + strlen("Failed to open "));
+            if (strstr(mplayer_output->str, "<") == NULL && strstr(mplayer_output->str, ">") == NULL
+                && idledata->streaming == FALSE)
+                error_msg =
+                    g_strdup_printf(_("Failed to open %s"),
+                                    mplayer_output->str + strlen("Failed to open "));
         }
     }
 
@@ -155,6 +168,12 @@ gboolean thread_reader_error(GIOChannel * source, GIOCondition condition, gpoint
         playback_error = ERROR_RETRY_WITH_PLAYLIST;
     }
 
+    if (strstr(mplayer_output->str, "MOV: missing header (moov/cmov) chunk") != NULL) {
+        idledata->retry_on_full_cache = TRUE;
+        g_strlcpy(idledata->progress_text, _("Delaying start until cache is full"), 1024);
+        g_idle_add(set_progress_text, idledata);
+    }
+
     if (strstr(mplayer_output->str, "Compressed SWF format not supported") != NULL) {
         error_msg = g_strdup_printf(_("Compressed SWF format not supported"));
     }
@@ -167,7 +186,8 @@ gboolean thread_reader_error(GIOChannel * source, GIOCondition condition, gpoint
         dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
                                         GTK_BUTTONS_CLOSE, error_msg);
         gtk_window_set_title(GTK_WINDOW(dialog), _("GNOME MPlayer Error"));
-        gtk_dialog_run(GTK_DIALOG(dialog));
+        if (control_id == 0)
+            gtk_dialog_run(GTK_DIALOG(dialog));
         gtk_widget_destroy(dialog);
         g_free(error_msg);
     }
@@ -188,13 +208,8 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
     GError *error = NULL;
     gchar *error_msg = NULL;
     GtkWidget *dialog;
-    gchar **parse;
-    gint name, artist;
-    gchar *cdname;
-    gchar *cdartist;
-    gchar *utf8name;
-    gchar *utf8artist;
     gdouble old_pos;
+    LangMenu *menu;
 
     if (source == NULL) {
         g_source_remove(watch_err_id);
@@ -211,10 +226,10 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         state = QUIT;
         g_source_remove(watch_err_id);
         g_source_remove(watch_in_hup_id);
-        g_mutex_unlock(thread_running);
+        //g_mutex_unlock(thread_running);
+		g_cond_signal(mplayer_complete_cond);
         return FALSE;
     }
-
     mplayer_output = g_string_new("");
 
     status = g_io_channel_read_line_string(source, mplayer_output, NULL, &error);
@@ -229,7 +244,8 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
                 g_string_free(mplayer_output, TRUE);
                 g_idle_add(set_stop, idledata);
                 state = QUIT;
-                g_mutex_unlock(thread_running);
+                //g_mutex_unlock(thread_running);
+				g_cond_signal(mplayer_complete_cond);
                 g_error_free(error);
                 error = NULL;
                 return FALSE;
@@ -238,14 +254,16 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
             g_string_free(mplayer_output, TRUE);
             g_idle_add(set_stop, idledata);
             state = QUIT;
-            g_mutex_unlock(thread_running);
+            //g_mutex_unlock(thread_running);
+			g_cond_signal(mplayer_complete_cond);
             error = NULL;
             return FALSE;
         }
     }
 
     if (strstr(mplayer_output->str, "ID_SIGNAL") != NULL) {
-        g_mutex_unlock(thread_running);
+        //g_mutex_unlock(thread_running);
+		g_cond_signal(mplayer_complete_cond);
         g_string_free(mplayer_output, TRUE);
         state = QUIT;
         return FALSE;
@@ -256,19 +274,27 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         return TRUE;
     }
     //printf("thread_reader state = %i : status = %i\n",state,status);
-    if (verbose && strstr(mplayer_output->str, "ANS_") == NULL)
+    if (verbose == 1 && strstr(mplayer_output->str, "ANS_") == NULL)
         printf("%s", mplayer_output->str);
 
+    // print out everything in really verbose mode
+    if (verbose > 1)
+        printf("thread reader: %s", mplayer_output->str);
 
     if ((strstr(mplayer_output->str, "(Quit)") != NULL)
         || (strstr(mplayer_output->str, "(End of file)") != NULL)) {
         state = QUIT;
         g_idle_add(set_stop, idledata);
         g_string_free(mplayer_output, TRUE);
-        g_mutex_unlock(thread_running);
+        //g_mutex_unlock(thread_running);
+		g_cond_signal(mplayer_complete_cond);	
         return FALSE;
     }
 
+    if (strstr(mplayer_output->str, "AO:") != NULL) {
+        idledata->audiopresent = TRUE;
+    }
+
     if (strstr(mplayer_output->str, "VO:") != NULL) {
         buf = strstr(mplayer_output->str, "VO:");
         sscanf(buf, "VO: [%9[^]]] %ix%i => %ix%i", vm, &actual_x, &actual_y, &play_x, &play_y);
@@ -281,12 +307,17 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
             printf("Resizing to %i x %i \n", actual_x, actual_y);
         idledata->width = actual_x;
         idledata->height = actual_y;
-        idledata->videopresent = 1;
+        idledata->videopresent = TRUE;
         g_idle_add(resize_window, idledata);
+        g_idle_add(set_subtitle_visibility, idledata);
         videopresent = 1;
         g_idle_add(set_volume_from_slider, NULL);
-        send_command("get_property metadata\n");
-        send_command("get_time_length\n");
+        if (idledata->length < 1.0)
+            send_command("get_time_length\n", TRUE);
+        send_command("get_property chapters\n", TRUE);
+        send_command("get_property sub_demux\n", TRUE);
+        // send_command("get_property switch_audio\n", TRUE);
+        send_command("pausing_keep_force get_property path\n", FALSE);
     }
 
     if (strstr(mplayer_output->str, "Video: no video") != NULL) {
@@ -296,11 +327,13 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         // printf("Resizing to %i x %i \n", actual_x, actual_y);
         idledata->width = actual_x;
         idledata->height = actual_y;
-        idledata->videopresent = 0;
+        idledata->videopresent = FALSE;
         g_idle_add(resize_window, idledata);
         g_idle_add(set_volume_from_slider, NULL);
-        send_command("get_property metadata\n");
-        send_command("get_time_length\n");
+        //send_command("get_property switch_audio\n", TRUE);
+        if (idledata->length < 1.0)
+            send_command("get_time_length\n", TRUE);
+        send_command("pausing_keep_force get_property path\n", FALSE);
     }
 
     if (strstr(mplayer_output->str, "ANS_PERCENT_POSITION") != 0) {
@@ -317,12 +350,18 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         g_idle_add(set_progress_time, idledata);
     }
 
+    if (strstr(mplayer_output->str, "ID_LENGTH") != 0) {
+        buf = strstr(mplayer_output->str, "ID_LENGTH");
+        sscanf(buf, "ID_LENGTH=%lf", &idledata->length);
+        g_idle_add(set_progress_time, idledata);
+    }
+
     if (strstr(mplayer_output->str, "ANS_TIME_POSITION") != 0) {
         old_pos = idledata->position;
         buf = strstr(mplayer_output->str, "ANS_TIME_POSITION");
         sscanf(buf, "ANS_TIME_POSITION=%lf", &idledata->position);
         if (idledata->position < old_pos) {
-            send_command("get_time_length\n");
+            send_command("get_time_length\n", FALSE);
             state = PLAYING;
         }
         g_idle_add(set_progress_time, idledata);
@@ -330,13 +369,13 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
             idledata->percent = idledata->position / idledata->length;
             g_idle_add(set_progress_value, idledata);
         } else {
-            send_command("get_percent_pos\n");
+            send_command("get_percent_pos\n", FALSE);
         }
     }
 
     if (strstr(mplayer_output->str, "ANS_stream_pos") != 0) {
         buf = strstr(mplayer_output->str, "ANS_stream_pos");
-        sscanf(buf, "ANS_stream_pos=%i", &idledata->byte_pos);
+        sscanf(buf, "ANS_stream_pos=%li", &idledata->byte_pos);
         g_idle_add(set_progress_time, idledata);
     }
 
@@ -351,6 +390,41 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         g_free(buf);
     }
 
+    if (strstr(mplayer_output->str, "ANS_chapters") != 0) {
+        buf = strstr(mplayer_output->str, "ANS_chapters");
+        sscanf(buf, "ANS_chapters=%i", &idledata->chapters);
+        if (idledata->chapters > 0)
+            idledata->has_chapters = TRUE;
+        g_idle_add(set_update_gui, NULL);
+    }
+
+    if (strstr(mplayer_output->str, "ANS_sub_visibility") != NULL) {
+        if (strstr(mplayer_output->str, "ANS_sub_visibility=yes")) {
+            idledata->sub_visible = TRUE;
+        } else {
+            idledata->sub_visible = FALSE;
+        }
+        g_idle_add(set_update_gui, NULL);
+    }
+
+    if (strstr(mplayer_output->str, "ANS_sub_demux") != 0) {
+        buf = strstr(mplayer_output->str, "ANS_sub_demux");
+        sscanf(buf, "ANS_sub_demux=%i", &idledata->sub_demux);
+        g_idle_add(set_update_gui, NULL);
+    }
+
+    if (strstr(mplayer_output->str, "ANS_switch_audio") != 0) {
+        buf = strstr(mplayer_output->str, "ANS_switch_audio");
+        sscanf(buf, "ANS_switch_audio=%i", &idledata->switch_audio);
+        g_idle_add(set_update_gui, NULL);
+    }
+/*
+    if (strstr(mplayer_output->str, "ID_AUDIO_ID") != 0) {
+        buf = strstr(mplayer_output->str, "ID_AUDIO_ID");
+        sscanf(buf, "ID_AUDIO_ID=%i", &idledata->switch_audio);
+        g_idle_add(set_update_gui, NULL);
+    }
+*/
     if (strstr(mplayer_output->str, "ANS_brightness") != 0) {
         buf = strstr(mplayer_output->str, "ANS_brightness");
         sscanf(buf, "ANS_brightness=%i", &idledata->brightness);
@@ -376,119 +450,15 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         sscanf(buf, "ANS_saturation=%i", &idledata->saturation);
     }
 
-    if (strstr(mplayer_output->str, "ANS_metadata") != 0) {
-        buf = strstr(mplayer_output->str, "ANS_metadata");
-        g_strlcpy(idledata->metadata, buf + strlen("ANS_metadata="), 1024);
-        g_strchomp(idledata->metadata);
-
-        if (idledata->metadata != NULL) {
-            parse = g_strsplit(idledata->metadata, ",", -1);
-            if (parse != NULL) {
-                i = 0;
-                artist = -1;
-                name = -1;
-                while (parse[i] != NULL) {
-                    if (g_strcasecmp(parse[i], "name") == 0 || g_strcasecmp(parse[i], "title") == 0) {
-                        name = i + 1;
-                    }
-                    if (g_strcasecmp(parse[i], "artist") == 0) {
-                        artist = i + 1;
-                    }
-                    i++;
-                }
-                if (name > 0 && artist > 0) {
-                    //message = g_strdup_printf("%s - %s",parse[name],parse[artist]);
-                    //printf("message = %s\n",message);
-                    //g_strlcpy(idledata->info, message, 1024);
-                    //g_free(message);
-                    //g_idle_add(set_media_info, idledata);
-                    utf8name = metadata_to_utf8(parse[name]);
-                    if (utf8name == NULL) {
-                        strip_unicode(parse[name], strlen(parse[name]));
-                        utf8name = g_strdup(parse[name]);
-                    }
-                    utf8artist = metadata_to_utf8(parse[artist]);
-                    if (utf8artist == NULL) {
-                        strip_unicode(parse[artist], strlen(parse[artist]));
-                        utf8artist = g_strdup(parse[artist]);
-                    }
-
-                    message =
-                        g_markup_printf_escaped(_
-                                                ("<small>\n<b>Title:</b>\t%s\n<b>Artist:</b>\t%s\n<b>File:</b>\t%s\n</small>"),
-                                                utf8name, utf8artist, idledata->info);
-                    g_strlcpy(idledata->media_info, message, 1024);
-                    g_free(message);
-                    g_free(utf8name);
-                    g_free(utf8artist);
-                    g_idle_add(set_media_label, idledata);
-                } else {
-                    if (g_strncasecmp(idledata->info, "cdda", 4) == 0
-                        && gtk_list_store_iter_is_valid(playliststore, &iter)) {
-                        gtk_tree_model_get(GTK_TREE_MODEL(playliststore), &iter, DESCRIPTION_COLUMN,
-                                           &cdname, ARTIST_COLUMN, &cdartist, -1);
-                        if (cdname != NULL) {
-                            utf8name = g_locale_to_utf8(cdname, -1, NULL, NULL, NULL);
-                            if (utf8name == NULL) {
-                                strip_unicode(cdname, strlen(cdname));
-                                utf8name = g_strdup(cdname);
-                            }
-                        } else {
-                            utf8name = g_strdup(_("Unknown"));
-                        }
-
-                        if (cdartist != NULL) {
-                            utf8artist = g_locale_to_utf8(cdartist, -1, NULL, NULL, NULL);
-                            if (utf8artist == NULL) {
-                                strip_unicode(cdartist, strlen(cdartist));
-                                utf8artist = g_strdup(cdartist);
-                            }
-                        } else {
-                            utf8artist = g_strdup(_("Unknown"));
-                        }
-
-                        message =
-                            g_markup_printf_escaped(_
-                                                    ("<small>\n<b>Title:</b>\t%s\n<b>Artist:</b>\t%s\n<b>Album:</b>\t%s\n</small>"),
-                                                    utf8name, utf8artist, playlistname);
-                        if (cdname != NULL) {
-                            g_free(cdname);
-                            cdname = NULL;
-                        }
-                        if (cdartist != NULL) {
-                            g_free(cdartist);
-                            cdartist = NULL;
-                        }
-                        g_free(utf8name);
-                        utf8name = NULL;
-                        g_free(utf8artist);
-                        utf8name = NULL;
-
-                    } else {
-                        message =
-                            g_markup_printf_escaped(_("<small>\n<b>File:</b>\t%s\n</small>"),
-                                                    idledata->info);
-                    }
-                    g_strlcpy(idledata->media_info, message, 1024);
-                    g_free(message);
-                    g_idle_add(set_media_label, idledata);
-                }
-                g_strfreev(parse);
-            }
-        } else {
-            message =
-                g_markup_printf_escaped(_("<small>\n<b>File:</b>\t%s\n</small>"), idledata->info);
-            g_strlcpy(idledata->media_info, message, 1024);
-            g_free(message);
-            g_idle_add(set_media_label, idledata);
-        }
+    if (strstr(mplayer_output->str, "ANS_path") != 0) {
+        if (verbose > 1)
+            printf("pausing keep force enabled\n");
+        use_pausing_keep_force = TRUE;
     }
 
     if (strstr(mplayer_output->str, "Cache fill") != 0) {
         buf = strstr(mplayer_output->str, "Cache fill");
         sscanf(buf, "Cache fill: %f%%", &percent);
-        //printf("Percent = %f\n",percent);
-        //printf("Buffer = %s\n",buf);
         buf = g_strdup_printf(_("Cache fill: %2.2f%%"), percent);
         g_strlcpy(idledata->progress_text, buf, 1024);
         g_free(buf);
@@ -554,9 +524,57 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
         g_strlcpy(idledata->audio_channels, buf, 16);
     }
 
+    if (strstr(mplayer_output->str, "ID_SID_") != 0) {
+        menu = g_new0(LangMenu, 1);
+        buf = strstr(mplayer_output->str, "ID_SID_");
+        sscanf(buf, "ID_SID_%i_", &menu->value);
+        g_string_truncate(mplayer_output, mplayer_output->len - 1);
+        buf = strstr(mplayer_output->str, "_LANG=");
+        if (buf != NULL) {
+            buf += strlen("_LANG=");
+            menu->label = g_strdup(buf);
+            g_idle_add(set_new_lang_menu, menu);
+        }
+    }
+
+    if (strstr(mplayer_output->str, "ID_SUBTITLE_ID=") != 0) {
+        menu = g_new0(LangMenu, 1);
+        buf = strstr(mplayer_output->str, "ID_SUBTITLE_ID");
+        sscanf(buf, "ID_SUBTITLE_ID=%i", &menu->value);
+        menu->label = g_strdup_printf("%i", menu->value);
+        g_idle_add(set_new_lang_menu, menu);
+    }
+
+    if (strstr(mplayer_output->str, "ID_AID_") != 0) {
+        menu = g_new0(LangMenu, 1);
+        buf = strstr(mplayer_output->str, "ID_AID_");
+        sscanf(buf, "ID_AID_%i_", &menu->value);
+        g_string_truncate(mplayer_output, mplayer_output->len - 1);
+        buf = strstr(mplayer_output->str, "_LANG=");
+        if (buf != NULL) {
+            buf += strlen("_LANG=");
+            menu->label = g_strdup(buf);
+            g_idle_add(set_new_audio_menu, menu);
+        }
+    }
+
+    if (strstr(mplayer_output->str, "ID_AUDIO_ID=") != 0) {
+        menu = g_new0(LangMenu, 1);
+        buf = strstr(mplayer_output->str, "ID_AUDIO_ID");
+        sscanf(buf, "ID_AUDIO_ID=%i", &menu->value);
+        menu->label = g_strdup_printf("%i", menu->value);
+        g_idle_add(set_new_audio_menu, menu);
+    }
+
     if (strstr(mplayer_output->str, "File not found") != 0) {
     }
 
+    if ((strstr(mplayer_output->str, "CHAPTERS") != 0)
+        && !(strstr(mplayer_output->str, "ID_CHAPTERS=0") != 0)) {
+        idledata->has_chapters = TRUE;
+        g_idle_add(set_update_gui, NULL);
+    }
+
     if (strstr(mplayer_output->str, "Couldn't open DVD device") != 0) {
         error_msg = g_strdup(mplayer_output->str);
     }
@@ -581,25 +599,29 @@ gboolean thread_reader(GIOChannel * source, GIOCondition condition, gpointer dat
             message = NULL;
         }
         if (buf != NULL) {
-            message = g_strdup_printf("%s", buf + 1);
-            for (i = 0; i < (int) strlen(message) - 1; i++) {
-                if (message[i] == '\'') {
-                    message[i] = '\0';
+            for (i = 1; i < (int) strlen(buf) - 1; i++) {
+                if (buf[i] == '\'') {
+                    buf[i] = '\0';
                     break;
                 }
             }
+            message = g_strdup_printf("<small>\n\t<big><b>%s</b></big>\n</small>", buf + 1);
         }
         if (message) {
-            g_strlcpy(idledata->info, message, 1024);
+            g_strlcpy(idledata->media_info, message, 1024);
+            g_free(message);
+            message = g_strdup_printf("\n\t<b>%s</b>\n", buf + 1);
+            g_strlcpy(idledata->media_notification, message, 1024);
             g_free(message);
             message = NULL;
         }
-        g_idle_add(set_media_info, idledata);
-    }
 
-    if (verbose > 1) {
-        printf("MPLAYER OUTPUT: %s\n", mplayer_output->str);
+
+        g_idle_add(set_media_label, idledata);
     }
+    //if (verbose > 1) {
+    //    printf("MPLAYER OUTPUT: %s\n", mplayer_output->str);
+    //}
 
     if (error_msg != NULL) {
         dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
@@ -630,7 +652,6 @@ gboolean thread_query(gpointer data)
     //              autopause
     // else
     // 
-
     if (data == NULL) {
         if (verbose)
             printf("shutting down threadquery since threaddata is NULL\n");
@@ -639,7 +660,8 @@ gboolean thread_query(gpointer data)
 
     if (threaddata->done == TRUE) {
         if (verbose)
-            printf("shutting down threadquery since threaddata->done is TRUE\n");
+            printf("shutting down threadquery for %s since threaddata->done is TRUE\n",
+                   threaddata->filename);
         g_free(threaddata);
         threaddata = NULL;
         return FALSE;
@@ -649,14 +671,14 @@ gboolean thread_query(gpointer data)
         // size = write(std_in, "get_percent_pos\n", strlen("get_percent_pos\n"));
         size = write(std_in, "get_time_pos\n", strlen("get_time_pos\n"));
         if (size == -1) {
-            shutdown();
+            mplayer_shutdown();
             return FALSE;
         } else {
 
             //send_command("get_time_pos\n");
-            send_command("get_property stream_pos\n");
+            send_command("get_property stream_pos\n", TRUE);
             if (threaddata->streaming)
-                send_command("get_property metadata\n");
+                send_command("get_property metadata\n", TRUE);
             g_idle_add(make_panel_and_mouse_invisible, NULL);
             return TRUE;
         }
@@ -678,45 +700,96 @@ gpointer launch_player(gpointer data)
     gint arg = 0;
     gchar *filename;
     gint count;
-    PlayData *p = (PlayData *) g_malloc(sizeof(PlayData));
+    PlayData *p = NULL;
     gchar *fontname;
     gchar *size;
+    gchar *buffer;
+    GError *error;
+    // GIOFlags flags;
+    GPid pid;
 
     ThreadData *threaddata = (ThreadData *) data;
 
     videopresent = 0;
     playback_error = NO_ERROR;
 
-    while (embed_window != -1 && !GTK_WIDGET_VISIBLE(window)) {
-        if (verbose)
-            printf("waiting for gui\n");
-    }
+    //while (embed_window != -1 && !GTK_WIDGET_VISIBLE(window)) {
+    //    if (verbose)
+    //        printf("waiting for gui\n");
+    //}
 
     g_mutex_lock(thread_running);
+#ifdef GIO_ENABLED
+    buffer = g_uri_unescape_string(threaddata->uri, NULL);
+#else
+    buffer = g_strdup(threaddata->uri);
+#endif
 
-    g_strlcpy(idledata->info, threaddata->filename, 1024);
+    g_strlcpy(idledata->info, buffer, 1024);
+    g_free(buffer);
     idledata->percent = 0.0;
     g_strlcpy(idledata->progress_text, "", 1024);
-    idledata->width = 1;
-    idledata->height = 1;
-    idledata->videopresent = 1;
     idledata->volume = volume;
     idledata->length = 0.0;
 
     g_idle_add(set_progress_value, idledata);
     g_idle_add(set_progress_text, idledata);
     g_idle_add(set_media_info, idledata);
-    //g_idle_add(set_window_visible, idledata);
+    g_idle_add(set_window_visible, idledata);
+    g_idle_add(resize_window, idledata);
 
     if (mplayer_bin == NULL || !g_file_test(mplayer_bin, G_FILE_TEST_EXISTS)) {
         argv[arg++] = g_strdup_printf("mplayer");
     } else {
         argv[arg++] = g_strdup_printf("%s", mplayer_bin);
     }
-    if (vo != NULL && strlen(vo) > 0) {
+
+    if ((vo != NULL && strlen(vo) > 0) || (ao != NULL && strlen(ao) > 0)) {
         argv[arg++] = g_strdup_printf("-profile");
         argv[arg++] = g_strdup_printf("gnome-mplayer");
     }
+
+    if (vo != NULL && g_ascii_strcasecmp(vo, "xvmc") == 0) {
+        if (g_strncasecmp(threaddata->filename, "dvd://", strlen("dvd://")) == 0
+            || g_strncasecmp(threaddata->filename, "dvdnav://", strlen("dvdnav://")) == 0) {
+            argv[arg++] = g_strdup_printf("-vc");
+            argv[arg++] = g_strdup_printf("ffmpeg12mc");
+        } else {
+            argv[arg++] = g_strdup_printf("-vo");
+            argv[arg++] = g_strdup_printf("xv");
+            if (!disable_deinterlace) {
+                argv[arg++] = g_strdup_printf("-vf-pre");
+                argv[arg++] = g_strdup_printf("yadif,softskip,scale");
+            }
+        }
+    }
+
+    if (vo != NULL && g_ascii_strcasecmp(vo, "vdpau") == 0) {
+        //printf("video_codec = '%s'\n",idledata->video_codec);
+        if (g_ascii_strcasecmp(idledata->video_codec, "ffmpeg1") == 0
+            || g_ascii_strcasecmp(idledata->video_codec, "ffmpeg2") == 0
+            || g_ascii_strcasecmp(idledata->video_codec, "mpegpes") == 0) {
+            argv[arg++] = g_strdup_printf("-vc");
+            argv[arg++] = g_strdup_printf("ffmpeg12vdpau");
+        } else if (g_ascii_strcasecmp(idledata->video_codec, "ffh264") == 0) {
+            argv[arg++] = g_strdup_printf("-vc");
+            argv[arg++] = g_strdup_printf("ffh264vdpau");
+        } else if (g_ascii_strcasecmp(idledata->video_codec, "ffwmv3") == 0) {
+            argv[arg++] = g_strdup_printf("-vc");
+            argv[arg++] = g_strdup_printf("ffwmv3vdpau");
+        } else if (g_ascii_strcasecmp(idledata->video_codec, "ffvc1") == 0) {
+            argv[arg++] = g_strdup_printf("-vc");
+            argv[arg++] = g_strdup_printf("ffvc1vdpau");
+        } else {
+            argv[arg++] = g_strdup_printf("-vo");
+            argv[arg++] = g_strdup_printf("xv");
+            if (!disable_deinterlace) {
+                argv[arg++] = g_strdup_printf("-vf-pre");
+                argv[arg++] = g_strdup_printf("yadif,softskip,scale");
+            }
+        }
+    }
+
     if (verbose < 2)
         argv[arg++] = g_strdup_printf("-quiet");
     argv[arg++] = g_strdup_printf("-slave");
@@ -726,12 +799,22 @@ gpointer launch_player(gpointer data)
     if (softvol)
         argv[arg++] = g_strdup_printf("-softvol");
 
+    if (mixer != NULL && strlen(mixer) > 0) {
+        if (ao == NULL || (ao != NULL && g_ascii_strncasecmp(ao, "alsa", 4) == 0)) {
+            argv[arg++] = g_strdup_printf("-mixer-channel");
+            argv[arg++] = g_strdup_printf("%s", mixer);
+        }
+    }
+
     if (!disable_framedrop)
         argv[arg++] = g_strdup_printf("-framedrop");
 
-    if (!disable_deinterlace) {
-        argv[arg++] = g_strdup_printf("-vf-pre");
-        argv[arg++] = g_strdup_printf("yadif,softskip,scale");
+    if (vo == NULL
+        || !(g_ascii_strcasecmp(vo, "xvmc") == 0 || g_ascii_strcasecmp(vo, "vdpau") == 0)) {
+        if (!disable_deinterlace) {
+            argv[arg++] = g_strdup_printf("-vf-pre");
+            argv[arg++] = g_strdup_printf("yadif,softskip,scale");
+        }
     }
 
     argv[arg++] = g_strdup_printf("-noconsolecontrols");
@@ -745,21 +828,22 @@ gpointer launch_player(gpointer data)
             // argv[arg++] = g_strdup_printf("-nocache");
         } else {
             argv[arg++] = g_strdup_printf("-nomouseinput");
-            if (threaddata->streaming && forcecache == FALSE) {
-                //argv[arg++] = g_strdup_printf("-user-agent");
-                //argv[arg++] = g_strdup_printf("NSPlayer");
-            } else {
+            if (threaddata->streaming || forcecache == TRUE || threaddata->playlist) {
                 argv[arg++] = g_strdup_printf("-cache");
                 argv[arg++] = g_strdup_printf("%i", cache_size);
+            } else {
+                argv[arg++] = g_strdup_printf("-nocache");
             }
         }
     }
+    // argv[arg++] = g_strdup_printf("-v");
     argv[arg++] = g_strdup_printf("-wid");
-    player_window = get_player_window();
+    player_window = idledata->windowid;
     argv[arg++] = g_strdup_printf("0x%x", player_window);
 
     if (control_id == 0) {
-        argv[arg++] = g_strdup_printf("-idx");
+//        if (!idledata->streaming)
+//            argv[arg++] = g_strdup_printf("-idx");
     } else {
         argv[arg++] = g_strdup_printf("-cookies");
     }
@@ -853,20 +937,33 @@ gpointer launch_player(gpointer data)
         g_strfreev(opts);
     }
 
-    argv[arg++] = g_strdup_printf("-vf-add");
-    argv[arg++] = g_strdup_printf("screenshot");
+    if (vo == NULL
+        || !(g_ascii_strcasecmp(vo, "xvmc") == 0 || g_ascii_strcasecmp(vo, "vdpau") == 0)) {
+        argv[arg++] = g_strdup_printf("-vf-add");
+        argv[arg++] = g_strdup_printf("screenshot");
+    }
 
     if (idledata->device != NULL) {
         argv[arg++] = g_strdup_printf("-dvd-device");
         argv[arg++] = g_strdup_printf("%s", idledata->device);
     }
 
-    if (playlist || threaddata->playlist)
+/*	This is needed when lavc is the default decoder, but mkv is the default as of 9/9/08
+	filename = g_utf8_strdown(threaddata->filename,-1);
+	if (strstr(filename,".mkv")) {
+        argv[arg++] = g_strdup_printf("-demuxer");
+        argv[arg++] = g_strdup_printf("mkv");
+	}		
+	g_free(filename);
+*/
+    if (threaddata->playlist)
         argv[arg++] = g_strdup_printf("-playlist");
+
+
     argv[arg] = g_strdup_printf("%s", threaddata->filename);
     argv[arg + 1] = NULL;
 
-    if (verbose > 1) {
+    if (verbose) {
         arg = 0;
         while (argv[arg] != NULL) {
             printf("%s ", argv[arg++]);
@@ -875,9 +972,16 @@ gpointer launch_player(gpointer data)
     }
 
     state = PAUSED;
+    error = NULL;
     ok = g_spawn_async_with_pipes(NULL, argv, NULL,
                                   G_SPAWN_SEARCH_PATH,
-                                  NULL, NULL, NULL, &std_in, &std_out, &std_err, NULL);
+                                  NULL, NULL, &pid, &std_in, &std_out, &std_err, &error);
+
+    if (error != NULL) {
+        printf("error code = %i - %s\n", error->code, error->message);
+        g_error_free(error);
+        error = NULL;
+    }
 
     arg = 0;
     while (argv[arg] != NULL) {
@@ -896,23 +1000,38 @@ gpointer launch_player(gpointer data)
             channel_in = NULL;
         }
 
+        if (channel_out != NULL) {
+            g_io_channel_unref(channel_out);
+            channel_out = NULL;
+        }
+
         if (channel_err != NULL) {
             g_io_channel_unref(channel_err);
             channel_err = NULL;
         }
 
-        channel_in = g_io_channel_unix_new(std_out);
+        channel_in = g_io_channel_unix_new(std_in);
+        channel_out = g_io_channel_unix_new(std_out);
         channel_err = g_io_channel_unix_new(std_err);
+/*        
+		flags = g_io_channel_get_flags(channel_in);
+        flags |= G_IO_FLAG_NONBLOCK;
+        g_io_channel_set_flags(channel_in, flags, NULL);
+        flags = g_io_channel_get_flags(channel_err);
+        flags |= G_IO_FLAG_NONBLOCK;
+        g_io_channel_set_flags(channel_err, flags, NULL);
+*/
         g_io_channel_set_close_on_unref(channel_in, TRUE);
+        g_io_channel_set_close_on_unref(channel_out, TRUE);
         g_io_channel_set_close_on_unref(channel_err, TRUE);
         watch_in_id =
-            g_io_add_watch_full(channel_in, G_PRIORITY_LOW, G_IO_IN | G_IO_HUP, thread_reader, NULL,
-                                NULL);
+            g_io_add_watch_full(channel_out, G_PRIORITY_LOW, G_IO_IN | G_IO_HUP, thread_reader,
+                                NULL, NULL);
         watch_err_id =
             g_io_add_watch_full(channel_err, G_PRIORITY_LOW, G_IO_IN | G_IO_ERR | G_IO_HUP,
                                 thread_reader_error, NULL, NULL);
         watch_in_hup_id =
-            g_io_add_watch_full(channel_in, G_PRIORITY_LOW, G_IO_ERR | G_IO_HUP, thread_complete,
+            g_io_add_watch_full(channel_out, G_PRIORITY_LOW, G_IO_ERR | G_IO_HUP, thread_complete,
                                 NULL, NULL);
 //        watch_in_id = g_io_add_watch(channel_in, G_IO_IN, thread_reader, NULL);
 //        watch_err_id = g_io_add_watch(channel_err, G_IO_IN | G_IO_ERR | G_IO_HUP, thread_reader_error, NULL);
@@ -924,10 +1043,15 @@ gpointer launch_player(gpointer data)
 #else
         g_timeout_add(1000, thread_query, threaddata);
 #endif
-
-        g_mutex_lock(thread_running);
+        if (verbose > 1)
+            printf
+                ("About to lock second time, next message about thread running, should be unlocking\n");
+        // g_mutex_lock(thread_running);
+		g_cond_wait(mplayer_complete_cond,thread_running);
         if (verbose)
             printf("Thread completing\n");
+        if (verbose > 1)
+            printf("thread running, in second lock\n");
         threaddata->done = TRUE;
         g_source_remove(watch_in_id);
         g_source_remove(watch_err_id);
@@ -959,9 +1083,30 @@ gpointer launch_player(gpointer data)
             g_io_channel_unref(channel_err);
             channel_err = NULL;
         }
+        close(std_in);
+        std_in = -1;
+        g_spawn_close_pid(pid);
+
+#ifdef GIO_ENABLED
+        if (idledata->tmpfile) {
+            if (verbose)
+                printf("removing temp file '%s'\n", threaddata->filename);
+            if (g_mutex_trylock(idledata->caching)) {
+                g_mutex_unlock(idledata->caching);
+            } else {
+                g_cancellable_cancel(idledata->cancel);
+            }
+            g_unlink(threaddata->filename);
+        }
+#endif
 
         dbus_enable_screensaver();
+
+        if (verbose > 1)
+            printf("thread running, before unlocking second time\n");
         g_mutex_unlock(thread_running);
+        if (verbose > 1)
+            printf("thread running, unlocked second time\n");
 
         if (dontplaynext == FALSE) {
             if (next_item_in_playlist(&iter)) {
@@ -970,6 +1115,7 @@ gpointer launch_player(gpointer data)
                                        COUNT_COLUMN, &count, PLAYLIST_COLUMN, &playlist, -1);
                     g_strlcpy(idledata->info, filename, 4096);
                     g_idle_add(set_media_info, idledata);
+                    p = (PlayData *) g_malloc(sizeof(PlayData));
                     g_strlcpy(p->filename, filename, 4096);
                     p->playlist = playlist;
                     g_idle_add(play, p);
@@ -985,11 +1131,16 @@ gpointer launch_player(gpointer data)
                                            &playlist, -1);
                         g_strlcpy(idledata->info, filename, 4096);
                         g_idle_add(set_media_info, idledata);
+                        p = (PlayData *) g_malloc(sizeof(PlayData));
                         g_strlcpy(p->filename, filename, 4096);
                         p->playlist = playlist;
                         g_idle_add(play, p);
                         g_free(filename);
                     }
+                } else {
+                    idledata->fullscreen = 0;
+                    g_idle_add(set_fullscreen, idledata);
+					g_idle_add(set_stop, idledata);
                 }
 
                 if (quit_on_complete) {
@@ -998,6 +1149,7 @@ gpointer launch_player(gpointer data)
             }
         } else {
             if (playback_error == ERROR_RETRY_WITH_PLAYLIST) {
+                p = (PlayData *) g_malloc(sizeof(PlayData));
                 g_strlcpy(p->filename, threaddata->filename, 4096);
                 p->playlist = 1;
                 g_idle_add(play, p);
@@ -1013,8 +1165,6 @@ gpointer launch_player(gpointer data)
 
     // printf("Thread done\n");
 
-
-
     thread = NULL;
     return NULL;
 }
